/****************************************************************************
* Biblioteka obslugi wyswietlacza graficznego, opartego na sterowniku
* SED1520 lub odpowiedniku.
* 
* Program demonstracyjny dla platformy ARM9 STR91XX.
*
* Nazwa pliku: demo_STR91.c
* Data: 20.08.2007
* Wersja: 1.0
*
* Copyright (C) 2007 by Rafal Baranowski.
*
* Niniejsze oprogramowanie udostepniane jest nieodplatnie i moze byc
* powielane, modyfikowane i wykorzystywane w celach komercyjnych pod
* warunkiem zachowania noty copyright.
* Na poprawne dzialanie i bezpieczenstwo tego oprogramowania nie jest
* udzalana zadna gwarancja.
****************************************************************************/

#include <SED1520.h>
#include <disp_conf.h>
#include <91x_map.h>
#include <91x_scu.h>

const uint8_t fonts[] = CHARSET;

void wait_ms(int ms) {
  while(ms--)
    for (uint8_t i=0; i<(50.0*F_CPU/25000000.0); i++)
      for (uint8_t j=0; j<35; j++)
        asm volatile("nop\n\t"::);
}

const uint8_t pict[] = {
   0x00, 0x00, 0xff, 0x03, 0xe0, 0x7f, 0x00, 0x00, 0x00, 0x80, 0xff, 0x07,
   0xf8, 0xff, 0x00, 0x00, 0x00, 0xc0, 0xff, 0x1f, 0xfc, 0xff, 0x00, 0x00,
   0x00, 0xc0, 0x0f, 0x3f, 0xfe, 0xf8, 0x01, 0x00, 0x00, 0xe0, 0x03, 0x3e,
   0x1f, 0xe0, 0x03, 0x00, 0x00, 0xf0, 0x01, 0x78, 0x0f, 0xc0, 0x03, 0x00,
   0x00, 0xf0, 0x70, 0x78, 0x8f, 0x87, 0x07, 0x00, 0x00, 0xf0, 0xf8, 0xf0,
   0xc7, 0x8f, 0x07, 0x00, 0x00, 0x78, 0xfc, 0xf1, 0xc7, 0x0f, 0x0f, 0x00,
   0x00, 0x78, 0xfc, 0xf1, 0xe7, 0x9f, 0x0f, 0x00, 0x00, 0x7c, 0xfc, 0xf1,
   0xc7, 0x0f, 0x0f, 0x00, 0x00, 0xfc, 0xf8, 0xf0, 0xc7, 0x8f, 0x0f, 0x00,
   0x00, 0xfc, 0x70, 0xf8, 0x07, 0x87, 0x1f, 0x00, 0x00, 0xfc, 0x01, 0x78,
   0x0f, 0xc0, 0x1f, 0x00, 0x00, 0xfc, 0x01, 0x3c, 0x1f, 0xe0, 0x1f, 0x00,
   0x00, 0xfc, 0x07, 0x3f, 0x7e, 0xf0, 0x1f, 0x00, 0x00, 0xfc, 0xff, 0x1f,
   0xfc, 0xff, 0x0f, 0x00, 0x00, 0xbc, 0xff, 0x0f, 0xf8, 0x7f, 0x1e, 0x00,
   0x00, 0x3c, 0xfe, 0x03, 0xf0, 0x3f, 0x0e, 0x00, 0x00, 0x38, 0xe8, 0x00,
   0x40, 0x0b, 0x0f, 0x00, 0x00, 0x38, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x00,
   0x00, 0x78, 0x18, 0x00, 0x00, 0x0e, 0x0f, 0x00, 0x00, 0x78, 0x3c, 0x00,
   0x00, 0x8e, 0x0f, 0x00, 0x00, 0xf0, 0x7c, 0x00, 0x00, 0x8f, 0x07, 0x00,
   0x00, 0xf0, 0xf8, 0x00, 0xc0, 0xcf, 0x07, 0x00, 0x00, 0xf0, 0xf1, 0x03,
   0xe0, 0xc7, 0x03, 0x00, 0x00, 0xe0, 0xe3, 0x0f, 0xfa, 0xe3, 0x03, 0x00,
   0x00, 0xc0, 0xc7, 0xff, 0xff, 0xf0, 0x01, 0x00, 0x00, 0x80, 0x0f, 0xff,
   0x7f, 0xf8, 0x00, 0x00, 0x00, 0x80, 0x1f, 0xfc, 0x1f, 0x7c, 0x00, 0x00,
   0x00, 0x00, 0x7f, 0x00, 0x00, 0x3f, 0x00, 0x00, 0x00, 0x00, 0xfc, 0x01,
   0xc0, 0x1f, 0x00, 0x00 };

int main(void) {
  Disp_init();
  Disp_set_ctempl_const(fonts);
  
  Disp_set_pxy(0,0);
  Disp_put_picture_const(pict,64,32);
  Disp_box(0,0,121,31,ON,NO);
  Disp_set_cxy(13,1);
  Disp_puts("Demo");
  Disp_set_cxy(12,2);
  Disp_puts("SED1520");
  Disp_update();
  
  wait_ms(3000);

  while (1) {
    for (uint8_t i=0; i<32; i++) {
      Disp_scroll_window(DOWN);
      wait_ms(150);
      Disp_update();
    }
    wait_ms(3000);
  }
  return 0;
}
